@@ -39,8 +39,6 @@
  * Management functions include:
  * - management includes initializing and shutting down all other sub-modules
  *   including: monitoring, allocation, log, cpuinfo and machine
- * - provide functions for safe access to PQoS API - this is required for
- *   allocation and monitoring modules which also implement PQoS API
  *
  * Capability functions:
  * - monitoring detection, this is to discover all monitoring event types.
@@ -56,6 +54,7 @@
 #include "api.h"
 #include "cpuinfo.h"
 #include "hw_cap.h"
+#include "lock.h"
 #include "log.h"
 #include "machine.h"
 #include "monitoring.h"
@@ -64,12 +63,8 @@
 #include "resctrl_alloc.h"
 #include "utils.h"
 
-#include <fcntl.h> /* O_CREAT */
-#include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/stat.h> /* S_Ixxx */
-#include <unistd.h>   /* usleep(), lockf() */
 
 /**
  * ---------------------------------------
@@ -77,15 +72,6 @@
  * ---------------------------------------
  */
 
-#ifndef LOCKFILE
-#ifdef __linux__
-#define LOCKFILE "/var/lock/libpqos"
-#endif
-#ifdef __FreeBSD__
-#define LOCKFILE "/var/tmp/libpqos.lockfile"
-#endif
-#endif /*!LOCKFILE*/
-
 #define PROC_CPUINFO "/proc/cpuinfo"
 
 /**
@@ -118,12 +104,6 @@ static const struct pqos_cpuinfo *m_cpu = NULL;
  */
 static int m_init_done = 0;
 
-/**
- * API thread/process safe access is secured through these locks.
- */
-static int m_apilock = -1;
-static pthread_mutex_t m_apilock_mutex;
-
 /**
  * Interface status
  *   0  PQOS_INTER_MSR
@@ -131,93 +111,23 @@ static pthread_mutex_t m_apilock_mutex;
  *   2  PQOS_INTER_OS_RESCTRL_MON
  */
 static enum pqos_interface m_interface = PQOS_INTER_MSR;
+
 /**
  * ---------------------------------------
- * Functions for safe multi-threading
+ * Local functions
  * ---------------------------------------
  */
 
-/**
- * @brief Initializes API locks
- *
- * @return Operation status
- * @retval 0 success
- * @retval -1 error
- */
-static int
-_pqos_api_init(void)
+enum pqos_interface
+_pqos_get_inter(void)
 {
-
-        const char *lock_filename = LOCKFILE;
-
-        if (m_apilock != -1)
-                return -1;
-
-        m_apilock = open(lock_filename, O_WRONLY | O_CREAT,
-                         S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-        if (m_apilock == -1)
-                return -1;
-
-        if (pthread_mutex_init(&m_apilock_mutex, NULL) != 0) {
-                close(m_apilock);
-                m_apilock = -1;
-                return -1;
-        }
-
-        return 0;
+        return m_interface;
 }
 
-/**
- * @brief Uninitializes API locks
- *
- * @return Operation status
- * @retval 0 success
- * @retval -1 error
- */
-static int
-_pqos_api_exit(void)
+PQOS_STATIC void
+_pqos_set_inter(const enum pqos_interface iface)
 {
-        int ret = 0;
-
-        if (close(m_apilock) != 0)
-                ret = -1;
-
-        if (pthread_mutex_destroy(&m_apilock_mutex) != 0)
-                ret = -1;
-
-        m_apilock = -1;
-
-        return ret;
-}
-
-void
-_pqos_api_lock(void)
-{
-        int err = 0;
-
-        if (lockf(m_apilock, F_LOCK, 0) != 0)
-                err = 1;
-
-        if (pthread_mutex_lock(&m_apilock_mutex) != 0)
-                err = 1;
-
-        if (err)
-                LOG_ERROR("API lock error!\n");
-}
-
-void
-_pqos_api_unlock(void)
-{
-        int err = 0;
-
-        if (lockf(m_apilock, F_ULOCK, 0) != 0)
-                err = 1;
-
-        if (pthread_mutex_unlock(&m_apilock_mutex) != 0)
-                err = 1;
-
-        if (err)
-                LOG_ERROR("API unlock error!\n");
+        m_interface = iface;
 }
 
 /**
@@ -252,7 +162,7 @@ _pqos_check_init(const int expect)
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
                   const struct pqos_cpuinfo *cpu,
                   const enum pqos_interface iface)
@@ -260,6 +170,9 @@ cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
         struct pqos_cap_l3ca *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_l3ca *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -297,7 +210,7 @@ cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
                   const struct pqos_cpuinfo *cpu,
                   const enum pqos_interface iface)
@@ -305,6 +218,9 @@ cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
         struct pqos_cap_l2ca *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_l2ca *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -342,7 +258,7 @@ cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_mba_discover(struct pqos_cap_mba **r_cap,
                  const struct pqos_cpuinfo *cpu,
                  const enum pqos_interface iface)
@@ -350,6 +266,9 @@ cap_mba_discover(struct pqos_cap_mba **r_cap,
         struct pqos_cap_mba *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_mba *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -390,7 +309,7 @@ cap_mba_discover(struct pqos_cap_mba **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 discover_capabilities(struct pqos_cap **p_cap,
                       const struct pqos_cpuinfo *cpu,
                       enum pqos_interface inter)
@@ -403,6 +322,8 @@ discover_capabilities(struct pqos_cap **p_cap,
         unsigned sz = 0;
         int ret = PQOS_RETVAL_RESOURCE;
 
+        if (p_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
         /**
          * Monitoring init
          */
@@ -586,8 +507,8 @@ error_exit:
  *
  * @return converted enumeration value into string
  */
-static const char *
-interface_to_string(enum pqos_interface interface)
+PQOS_STATIC const char *
+_cap_interface_to_string(enum pqos_interface interface)
 {
         switch (interface) {
         case PQOS_INTER_MSR:
@@ -612,15 +533,17 @@ interface_to_string(enum pqos_interface interface)
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 discover_interface(enum pqos_interface requested_interface,
                    enum pqos_interface *interface)
 {
         char *environment = NULL;
 
         LOG_INFO("Requested interface: %s\n",
-                 interface_to_string(requested_interface));
+                 _cap_interface_to_string(requested_interface));
 
+        if (interface == NULL)
+                return PQOS_RETVAL_PARAM;
 #ifdef __linux__
         if (requested_interface != PQOS_INTER_MSR &&
             requested_interface != PQOS_INTER_OS &&
@@ -672,7 +595,8 @@ discover_interface(enum pqos_interface requested_interface,
                 *interface = requested_interface;
         }
 
-        LOG_INFO("Selected interface: %s\n", interface_to_string(*interface));
+        LOG_INFO("Selected interface: %s\n",
+                 _cap_interface_to_string(*interface));
         return PQOS_RETVAL_OK;
 }
 
@@ -696,16 +620,17 @@ pqos_init(const struct pqos_config *config)
         if (config == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        if (_pqos_api_init() != 0) {
+        if (lock_init() != 0) {
                 fprintf(stderr, "API lock initialization error!\n");
                 return PQOS_RETVAL_ERROR;
         }
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(0);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
+                lock_fini();
                 return ret;
         }
 
@@ -774,11 +699,11 @@ pqos_init(const struct pqos_config *config)
 
         ret = api_init(interface, m_cpu->vendor);
         if (ret != PQOS_RETVAL_OK) {
-                LOG_ERROR("_pqos_api_init() error %d\n", ret);
+                LOG_ERROR("lock_init() error %d\n", ret);
                 goto machine_init_error;
         }
 
-        m_interface = interface;
+        _pqos_set_inter(interface);
 
         ret = pqos_alloc_init(m_cpu, m_cap, config);
         switch (ret) {
@@ -844,10 +769,10 @@ init_error:
         if (ret == PQOS_RETVAL_OK)
                 m_init_done = 1;
 
-        _pqos_api_unlock();
+        lock_release();
 
         if (ret != PQOS_RETVAL_OK)
-                _pqos_api_exit();
+                lock_fini();
 
         return ret;
 }
@@ -859,12 +784,12 @@ pqos_fini(void)
         int retval = PQOS_RETVAL_OK;
         unsigned i = 0;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
-                _pqos_api_exit();
+                lock_release();
+                lock_fini();
                 return ret;
         }
 
@@ -889,16 +814,18 @@ pqos_fini(void)
 
         m_cpu = NULL;
 
-        for (i = 0; i < m_cap->num_cap; i++)
-                free(m_cap->capabilities[i].u.generic_ptr);
-        free((void *)m_cap);
-        m_cap = NULL;
+        if (m_cap != NULL) {
+                for (i = 0; i < m_cap->num_cap; i++)
+                        free(m_cap->capabilities[i].u.generic_ptr);
+                free((void *)m_cap);
+                m_cap = NULL;
+        };
 
         m_init_done = 0;
 
-        _pqos_api_unlock();
+        lock_release();
 
-        if (_pqos_api_exit() != 0)
+        if (lock_fini() != 0)
                 retval = PQOS_RETVAL_ERROR;
 
         return retval;
@@ -922,11 +849,11 @@ pqos_cap_get(const struct pqos_cap **cap, const struct pqos_cpuinfo **cpu)
         if (cap == NULL && cpu == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
@@ -939,7 +866,7 @@ pqos_cap_get(const struct pqos_cap **cap, const struct pqos_cpuinfo **cpu)
                 ASSERT(*cpu != NULL);
         }
 
-        _pqos_api_unlock();
+        lock_release();
         return PQOS_RETVAL_OK;
 }
 
@@ -1141,22 +1068,16 @@ pqos_inter_get(enum pqos_interface *interface)
         if (interface == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
         *interface = _pqos_get_inter();
 
-        _pqos_api_unlock();
+        lock_release();
         return PQOS_RETVAL_OK;
 }
-
-enum pqos_interface
-_pqos_get_inter(void)
-{
-        return m_interface;
-}
