@@ -1,8 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2017 Intel Corporation. All rights reserved.
- * All rights reserved.
+ * Copyright(c) 2014-2022 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,35 +37,103 @@
  * These functions need no synchronization mechanisms.
  *
  */
+
+#include "utils.h"
+
+#include "cpuinfo.h"
+#include "pqos.h"
+
 #include <stdlib.h>
 #include <string.h>
-#ifdef __linux__
-#include <alloca.h>       /* alloca() */
-#endif /* __linux__ */
-
-#include "pqos.h"
-#include "types.h"
-#include "utils.h"
 
 #define TOPO_OBJ_SOCKET     0
 #define TOPO_OBJ_L2_CLUSTER 2
 #define TOPO_OBJ_L3_CLUSTER 3
 
-static int m_interface = PQOS_INTER_MSR;
-
-
 int
 _pqos_utils_init(int interface)
 {
-    m_interface = interface;
+        UNUSED_PARAM(interface);
 
-    return PQOS_RETVAL_OK;
+        return PQOS_RETVAL_OK;
 }
 
+unsigned *
+pqos_cpu_get_mba_ids(const struct pqos_cpuinfo *cpu, unsigned *count)
+{
+        unsigned mba_id_count = 0, i = 0;
+        unsigned *mba_ids = NULL;
+
+        ASSERT(cpu != NULL);
+        ASSERT(count != NULL);
+        if (cpu == NULL || count == NULL)
+                return NULL;
+
+        mba_ids = (unsigned *)malloc(sizeof(mba_ids[0]) * cpu->num_cores);
+        if (mba_ids == NULL)
+                return NULL;
+
+        for (i = 0; i < cpu->num_cores; i++) {
+                unsigned j = 0;
+
+                /**
+                 * Check if this mba id is already on the \a mbas list
+                 */
+                for (j = 0; j < mba_id_count && mba_id_count > 0; j++)
+                        if (cpu->cores[i].mba_id == mba_ids[j])
+                                break;
+
+                if (j >= mba_id_count || mba_id_count == 0) {
+                        /**
+                         * This mba_id wasn't reported before
+                         */
+                        mba_ids[mba_id_count++] = cpu->cores[i].mba_id;
+                }
+        }
+
+        *count = mba_id_count;
+        return mba_ids;
+}
 
 unsigned *
-pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu,
-                     unsigned *count)
+pqos_cpu_get_l3cat_ids(const struct pqos_cpuinfo *cpu, unsigned *count)
+{
+        unsigned l3cat_count = 0, i = 0;
+        unsigned *l3cat_ids = NULL;
+
+        ASSERT(cpu != NULL);
+        ASSERT(count != NULL);
+        if (cpu == NULL || count == NULL)
+                return NULL;
+
+        l3cat_ids = (unsigned *)malloc(sizeof(l3cat_ids[0]) * cpu->num_cores);
+        if (l3cat_ids == NULL)
+                return NULL;
+
+        for (i = 0; i < cpu->num_cores; i++) {
+                unsigned j = 0;
+
+                /**
+                 * Check if this l3cat id is already on the \a l3cat_ids list
+                 */
+                for (j = 0; j < l3cat_count && l3cat_count > 0; j++)
+                        if (cpu->cores[i].l3cat_id == l3cat_ids[j])
+                                break;
+
+                if (j >= l3cat_count || l3cat_count == 0) {
+                        /**
+                         * This l3cat_id wasn't reported before
+                         */
+                        l3cat_ids[l3cat_count++] = cpu->cores[i].l3cat_id;
+                }
+        }
+
+        *count = l3cat_count;
+        return l3cat_ids;
+}
+
+unsigned *
+pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu, unsigned *count)
 {
         unsigned scount = 0, i = 0;
         unsigned *sockets = NULL;
@@ -76,7 +143,7 @@ pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu,
         if (cpu == NULL || count == NULL)
                 return NULL;
 
-        sockets = (unsigned *) malloc(sizeof(sockets[0]) * cpu->num_cores);
+        sockets = (unsigned *)malloc(sizeof(sockets[0]) * cpu->num_cores);
         if (sockets == NULL)
                 return NULL;
 
@@ -103,8 +170,7 @@ pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu,
 }
 
 unsigned *
-pqos_cpu_get_l2ids(const struct pqos_cpuinfo *cpu,
-                   unsigned *count)
+pqos_cpu_get_l2ids(const struct pqos_cpuinfo *cpu, unsigned *count)
 {
         unsigned l2count = 0, i = 0;
         unsigned *l2ids = NULL;
@@ -114,7 +180,7 @@ pqos_cpu_get_l2ids(const struct pqos_cpuinfo *cpu,
         if (cpu == NULL || count == NULL)
                 return NULL;
 
-        l2ids = (unsigned *) malloc(sizeof(l2ids[0]) * cpu->num_cores);
+        l2ids = (unsigned *)malloc(sizeof(l2ids[0]) * cpu->num_cores);
         if (l2ids == NULL)
                 return NULL;
 
@@ -168,7 +234,7 @@ __get_cores_per_topology_obj(const struct pqos_cpuinfo *cpu,
         if (cpu == NULL || count == NULL)
                 return NULL;
 
-        core_list = (unsigned *) malloc(cpu->num_cores * sizeof(core_list[0]));
+        core_list = (unsigned *)malloc(cpu->num_cores * sizeof(core_list[0]));
         if (core_list == NULL)
                 return NULL;
 
@@ -190,7 +256,8 @@ __get_cores_per_topology_obj(const struct pqos_cpuinfo *cpu,
 }
 
 unsigned *
-pqos_cpu_get_cores_l3id(const struct pqos_cpuinfo *cpu, const unsigned l3_id,
+pqos_cpu_get_cores_l3id(const struct pqos_cpuinfo *cpu,
+                        const unsigned l3_id,
                         unsigned *count)
 {
         return __get_cores_per_topology_obj(cpu, TOPO_OBJ_L3_CLUSTER, l3_id,
@@ -211,7 +278,7 @@ pqos_cpu_get_cores(const struct pqos_cpuinfo *cpu,
         if (cpu == NULL || count == NULL)
                 return NULL;
 
-        cores = (unsigned *) malloc(cpu->num_cores * sizeof(cores[0]));
+        cores = (unsigned *)malloc(cpu->num_cores * sizeof(cores[0]));
         if (cores == NULL)
                 return NULL;
 
@@ -267,6 +334,50 @@ pqos_cpu_get_one_core(const struct pqos_cpuinfo *cpu,
         return PQOS_RETVAL_ERROR;
 }
 
+int
+pqos_cpu_get_one_by_l3cat_id(const struct pqos_cpuinfo *cpu,
+                             const unsigned l3cat_id,
+                             unsigned *lcore)
+{
+        unsigned i = 0;
+
+        ASSERT(cpu != NULL);
+        ASSERT(lcore != NULL);
+
+        if (cpu == NULL || lcore == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        for (i = 0; i < cpu->num_cores; i++)
+                if (cpu->cores[i].l3cat_id == l3cat_id) {
+                        *lcore = cpu->cores[i].lcore;
+                        return PQOS_RETVAL_OK;
+                }
+
+        return PQOS_RETVAL_ERROR;
+}
+
+int
+pqos_cpu_get_one_by_mba_id(const struct pqos_cpuinfo *cpu,
+                           const unsigned mba_id,
+                           unsigned *lcore)
+{
+        unsigned i = 0;
+
+        ASSERT(cpu != NULL);
+        ASSERT(lcore != NULL);
+
+        if (cpu == NULL || lcore == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        for (i = 0; i < cpu->num_cores; i++)
+                if (cpu->cores[i].mba_id == mba_id) {
+                        *lcore = cpu->cores[i].lcore;
+                        return PQOS_RETVAL_OK;
+                }
+
+        return PQOS_RETVAL_ERROR;
+}
+
 int
 pqos_cpu_get_one_by_l2id(const struct pqos_cpuinfo *cpu,
                          const unsigned l2id,
@@ -290,8 +401,7 @@ pqos_cpu_get_one_by_l2id(const struct pqos_cpuinfo *cpu,
 }
 
 int
-pqos_cpu_check_core(const struct pqos_cpuinfo *cpu,
-                    const unsigned lcore)
+pqos_cpu_check_core(const struct pqos_cpuinfo *cpu, const unsigned lcore)
 {
         unsigned i = 0;
 
@@ -356,6 +466,8 @@ pqos_cap_get_type(const struct pqos_cap *cap,
         if (cap == NULL || cap_item == NULL)
                 return PQOS_RETVAL_PARAM;
 
+        *cap_item = NULL;
+
         ASSERT(type < PQOS_CAP_TYPE_NUMOF);
         if (type >= PQOS_CAP_TYPE_NUMOF)
                 return PQOS_RETVAL_PARAM;
@@ -363,12 +475,6 @@ pqos_cap_get_type(const struct pqos_cap *cap,
         for (i = 0; i < cap->num_cap; i++) {
                 if (cap->capabilities[i].type != type)
                         continue;
-                /**
-                 * Feature not supported for OS interface
-                 */
-                if (m_interface == PQOS_INTER_OS &&
-                    !cap->capabilities[i].os_support)
-                        continue;
                 *cap_item = &cap->capabilities[i];
                 ret = PQOS_RETVAL_OK;
                 break;
@@ -402,11 +508,6 @@ pqos_cap_get_event(const struct pqos_cap *cap,
         for (i = 0; i < mon->num_events; i++) {
                 if (mon->events[i].type != event)
                         continue;
-                /**
-                 * Feature not supported for OS interface
-                 */
-                if (m_interface == PQOS_INTER_OS && !mon->events[i].os_support)
-                        continue;
 
                 *p_mon = &mon->events[i];
                 ret = PQOS_RETVAL_OK;
@@ -417,8 +518,7 @@ pqos_cap_get_event(const struct pqos_cap *cap,
 }
 
 int
-pqos_l3ca_get_cos_num(const struct pqos_cap *cap,
-                      unsigned *cos_num)
+pqos_l3ca_get_cos_num(const struct pqos_cap *cap, unsigned *cos_num)
 {
         const struct pqos_capability *item = NULL;
         int ret = PQOS_RETVAL_OK;
@@ -429,7 +529,7 @@ pqos_l3ca_get_cos_num(const struct pqos_cap *cap,
 
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_L3CA, &item);
         if (ret != PQOS_RETVAL_OK)
-                return ret;                           /**< no L3CA capability */
+                return ret; /**< no L3CA capability */
 
         ASSERT(item != NULL);
         *cos_num = item->u.l3ca->num_classes;
@@ -437,8 +537,7 @@ pqos_l3ca_get_cos_num(const struct pqos_cap *cap,
 }
 
 int
-pqos_l2ca_get_cos_num(const struct pqos_cap *cap,
-                      unsigned *cos_num)
+pqos_l2ca_get_cos_num(const struct pqos_cap *cap, unsigned *cos_num)
 {
         const struct pqos_capability *item = NULL;
         int ret = PQOS_RETVAL_OK;
@@ -449,7 +548,7 @@ pqos_l2ca_get_cos_num(const struct pqos_cap *cap,
 
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_L2CA, &item);
         if (ret != PQOS_RETVAL_OK)
-                return ret;                           /**< no L2CA capability */
+                return ret; /**< no L2CA capability */
 
         ASSERT(item != NULL);
         *cos_num = item->u.l2ca->num_classes;
@@ -457,8 +556,7 @@ pqos_l2ca_get_cos_num(const struct pqos_cap *cap,
 }
 
 int
-pqos_mba_get_cos_num(const struct pqos_cap *cap,
-                      unsigned *cos_num)
+pqos_mba_get_cos_num(const struct pqos_cap *cap, unsigned *cos_num)
 {
         const struct pqos_capability *item = NULL;
         int ret = PQOS_RETVAL_OK;
@@ -469,7 +567,7 @@ pqos_mba_get_cos_num(const struct pqos_cap *cap,
 
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MBA, &item);
         if (ret != PQOS_RETVAL_OK)
-                return ret;                           /**< no MBA capability */
+                return ret; /**< no MBA capability */
 
         ASSERT(item != NULL);
         *cos_num = item->u.mba->num_classes;
@@ -490,12 +588,78 @@ pqos_l3ca_cdp_enabled(const struct pqos_cap *cap,
 
         ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_L3CA, &item);
         if (ret != PQOS_RETVAL_OK)
-                return ret;                           /**< no L3CA capability */
+                return ret; /**< no L3CA capability */
 
         ASSERT(item != NULL);
         if (cdp_supported != NULL)
                 *cdp_supported = item->u.l3ca->cdp;
+
         if (cdp_enabled != NULL)
                 *cdp_enabled = item->u.l3ca->cdp_on;
+
         return ret;
 }
+
+int
+pqos_l2ca_cdp_enabled(const struct pqos_cap *cap,
+                      int *cdp_supported,
+                      int *cdp_enabled)
+{
+        const struct pqos_capability *l2ca = NULL;
+        int ret = PQOS_RETVAL_OK;
+
+        ASSERT(cap != NULL && (cdp_enabled != NULL || cdp_supported != NULL));
+        if (cap == NULL || (cdp_enabled == NULL && cdp_supported == NULL))
+                return PQOS_RETVAL_PARAM;
+
+        ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_L2CA, &l2ca);
+        if (ret != PQOS_RETVAL_OK)
+                return ret; /**< no L2CA capability */
+
+        ASSERT(l2ca != NULL);
+        if (cdp_supported != NULL)
+                *cdp_supported = l2ca->u.l2ca->cdp;
+
+        if (cdp_enabled != NULL)
+                *cdp_enabled = l2ca->u.l2ca->cdp_on;
+
+        return ret;
+}
+
+int
+pqos_mba_ctrl_enabled(const struct pqos_cap *cap,
+                      int *ctrl_supported,
+                      int *ctrl_enabled)
+{
+        const struct pqos_capability *mba_cap = NULL;
+        int ret = PQOS_RETVAL_OK;
+
+        ASSERT(cap != NULL && (ctrl_supported != NULL || ctrl_enabled != NULL));
+        if (cap == NULL || (ctrl_supported == NULL && ctrl_enabled == NULL))
+                return PQOS_RETVAL_PARAM;
+
+        ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MBA, &mba_cap);
+        if (ret != PQOS_RETVAL_OK)
+                return ret; /**< no MBA capability */
+
+        ASSERT(mba_cap != NULL);
+        if (ctrl_supported != NULL)
+                *ctrl_supported = mba_cap->u.mba->ctrl;
+
+        if (ctrl_enabled != NULL)
+                *ctrl_enabled = mba_cap->u.mba->ctrl_on;
+
+        return PQOS_RETVAL_OK;
+}
+
+enum pqos_vendor
+pqos_get_vendor(const struct pqos_cpuinfo *cpu)
+{
+        return cpu->vendor;
+}
+
+void
+pqos_free(void *ptr)
+{
+        free(ptr);
+}
