@@ -1,8 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2017 Intel Corporation. All rights reserved.
- * All rights reserved.
+ * Copyright(c) 2014-2022 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,10 +34,6 @@
  * Allocation module
  */
 
-#include <stdint.h>
-#include <stdio.h>
-#include "pqos.h"
-
 #ifndef __ALLOCATION_H__
 #define __ALLOCATION_H__
 
@@ -46,6 +41,11 @@
 extern "C" {
 #endif
 
+#include "pqos.h"
+
+#include <stdint.h>
+#include <stdio.h>
+
 /**
  * @brief Defines allocation class of service
  *
@@ -67,8 +67,6 @@ void selfn_allocation_assoc(const char *arg);
  * @param [in] cap_l3ca L3 CAT capability structures
  * @param [in] cap_l2ca L2 CAT capability structures
  * @param [in] cap_mba MBA capability structures
- * @param [in] sock_count number of detected CPU sockets
- * @param [in] sockets arrays with detected CPU socket id's
  * @param [in] cpu_info cpu information structure
  * @param [in] verbose verbose mode flag
  */
@@ -76,8 +74,6 @@ void alloc_print_config(const struct pqos_capability *cap_mon,
                         const struct pqos_capability *cap_l3ca,
                         const struct pqos_capability *cap_l2ca,
                         const struct pqos_capability *cap_mba,
-                        const unsigned sock_count,
-                        const unsigned *sockets,
                         const struct pqos_cpuinfo *cpu_info,
                         const int verbose);
 
@@ -100,7 +96,6 @@ int alloc_apply(const struct pqos_capability *cap_l3ca,
                 const struct pqos_capability *cap_mba,
                 const struct pqos_cpuinfo *cpu);
 
-
 #ifdef __cplusplus
 }
 #endif
